/**
 * Jarapac DCE/RPC Framework
 * 
 * Copyright (C) 2007-2008 StillSecure www.stillsecure.com
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Class: winreg.java
 * 
 * Author: Joe Webster
 * 
 * Description: provides winreg (registry) services.
 */
package msrpc;

import rpc.*;
import ndr.*;

/**
 * This class was originally generated via the midlc compiler from IDL code
 * provided with Jarapac and Samba 4.0.0tp4. It has been significantly altered
 * for for this implementation.
 * 
 * @author Joe Webster
 * 
 */
public class winreg extends Stub {

    /**
     * winreg pipe defines
     */
    public static final int REG_OPEN_HKCR = 0x00;
    public static final int REG_OPEN_HKCU = 0x01;
    public static final int REG_OPEN_HKLM = 0x02;
    public static final int REG_OPEN_HKPD = 0x03;
    public static final int REG_OPEN_HKU = 0x04;
    public static final int REG_CLOSE = 0x05;
    public static final int REG_CREATE_KEY = 0x06;
    public static final int REG_DELETE_KEY = 0x07;
    public static final int REG_DELETE_VALUE = 0x08;
    public static final int REG_ENUM_KEY = 0x09;
    public static final int REG_ENUM_VALUE = 0x0a;
    public static final int REG_FLUSH_KEY = 0x0B;
    public static final int REG_GET_KEY_SEC = 0x0c;
    public static final int REG_OPEN_ENTRY = 0x0f;
    public static final int REG_QUERY_KEY = 0x10;
    public static final int REG_INFO = 0x11;
    public static final int REG_SET_KEY_SEC = 0x15;
    public static final int REG_CREATE_VALUE = 0x16;
    public static final int REG_SHUTDOWN = 0x18;
    public static final int REG_ABORT_SHUTDOWN = 0x19;
    public static final int REG_UNK_1A = 0x1a;
    public static final int REG_QUERY_MULTI_VAL = 0x1d;
    public static final int REG_SHUTDOWN2 = 0x1e;

    public static final long INFINITY = 0x7fffffffffffffffL;

    /**
     * Registry data type defines
     */
    public static final int REG_NONE = 0x00;
    public static final int REG_SZ = 0x01;
    public static final int REG_EXPAND_SZ = 0x02;
    public static final int REG_BINARY = 0x03;
    public static final int REG_DWORD = 0x04;
    public static final int REG_DWORD_BIG_ENDIAN = 0x05;
    public static final int REG_LINK = 0x06;
    public static final int REG_MULTI_SZ = 0x07;
    public static final int REG_RESOURCE_LIST = 0x08;

    /**
     * registry permission masks
     */
    public static final int REG_ACC_MAX_ALLOWED = 0x02000000;
    public static final int REG_ACC_READ_ONLY = 0x00020019;

    /**
     * error codes
     */
    public static final int WERR_MORE_DATA = 0xEA;
    public static final int WERR_NO_MORE_ITEMS = 0x103;

    public static final String MY_SYNTAX = "338cd001-2244-31f1-aaaa-900038001003:1.0";

    public String getSyntax() {
        return MY_SYNTAX;
    }

    /**
     * processing for an anonymous registry entry
     */
    public static class RegOpenUnknown extends NdrObject {

        public short unknown0;
        public short unknown1;

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            _dst.align(2);
            _dst.enc_ndr_short(unknown0);
            _dst.enc_ndr_short(unknown1);

        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            _src.align(2);
            unknown0 = (short) _src.dec_ndr_short();
            unknown1 = (short) _src.dec_ndr_short();

        }
    }

    /**
     * base class for all winreg commands. defines properties common to all.
     * 
     * Properties:
     * 
     * retval - return value. If the function succeeds, the return value is
     * ERROR_SUCCESS. If the function fails, the return value is a system error
     * code. If the lpData buffer is too small to receive the data, the function
     * returns ERROR_MORE_DATA.
     * 
     * handle - A handle to an open registry key. The key must have been opened
     * with the KEY_ENUMERATE_SUB_KEYS access right. For more information, see
     * Registry Key Security and Access Rights. This handle is returned by
     * RegCreateKey, RegCreateKeyTransacted, RegOpenKey, or
     * RegOpenKeyTransacted.
     * 
     * @author Joe Webster
     * 
     */
    public static abstract class RegCmd extends NdrObject {

        public int retval;
        public rpc.policy_handle handle;
    }

    /**
     * Base class for all "open hive key" command classes Defines encode and
     * decode methods.
     * 
     * @author jwebster
     * 
     */
    public static abstract class RegOpenHK extends RegCmd {

        public int retval;
        public RegOpenUnknown unknown;
        public int access_mask;

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            _dst.enc_ndr_referent(unknown, 1);
            if (unknown != null) {
                unknown.encode(_ndr, _dst);
            }
            _dst.enc_ndr_long(access_mask);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            handle.decode(_ndr, _src);
            retval = (int) _src.dec_ndr_long();
        }
    }

    /**
     * Open Classes Root hive key class
     */
    public static class RegOpenHKCR extends RegOpenHK {

        public int getOpnum() {
            return REG_OPEN_HKCR;
        }

        public RegOpenHKCR(RegOpenUnknown unknown,
                           int access_mask,
                           rpc.policy_handle handle) {
            this.unknown = unknown;
            this.access_mask = access_mask;
            this.handle = handle;
        }
    }

    /**
     * Open Current User hive key class
     */
    public static class RegOpenHKCU extends RegOpenHK {

        public int getOpnum() {
            return REG_OPEN_HKCU;
        }

        public RegOpenHKCU(RegOpenUnknown unknown,
                           int access_mask,
                           rpc.policy_handle handle) {
            this.unknown = unknown;
            this.access_mask = access_mask;
            this.handle = handle;
        }
    }

    /**
     * Open Local Machine hive key class
     */
    public static class RegOpenHKLM extends RegOpenHK {

        public int getOpnum() {
            return REG_OPEN_HKLM;
        }

        public RegOpenHKLM(RegOpenUnknown unknown,
                           int access_mask,
                           rpc.policy_handle handle) {
            this.unknown = unknown;
            this.access_mask = access_mask;
            this.handle = handle;
        }
    }

    /**
     * Open Users hive key class
     */
    public static class RegOpenHKU extends RegOpenHK {

        public int getOpnum() {
            return REG_OPEN_HKU;
        }

        public RegOpenHKU(RegOpenUnknown unknown,
                          int access_mask,
                          rpc.policy_handle handle) {
            this.unknown = unknown;
            this.access_mask = access_mask;
            this.handle = handle;
        }
    }

    /**
     * Open Performance Data Root hive key class
     */
    public static class RegOpenHKPD extends RegOpenHK {

        public int getOpnum() {
            return REG_OPEN_HKPD;
        }

        public RegOpenHKPD(RegOpenUnknown unknown,
                           int access_mask,
                           rpc.policy_handle handle) {
            this.unknown = unknown;
            this.access_mask = access_mask;
            this.handle = handle;
        }
    }

    /**
     * Open registry key class
     * 
     * Properties: subKey - The name of the registry subkey to be opened.
     * 
     * options - This parameter is reserved and must be zero.
     * 
     * desired - A mask that specifies the desired access rights to the key. The
     * function fails if the security descriptor of the key does not permit the
     * requested access for the calling process. For more information, see
     * Registry Key Security and Access Rights.
     * 
     * result - pointer to a variable that receives a handle to the opened key.
     * If the key is not one of the predefined registry keys, call the
     * RegCloseKey function after you have finished using the handle.
     */
    public static class RegOpenKey extends RegCmd {

        public int getOpnum() {
            return REG_OPEN_ENTRY;
        }

        public rpc.policy_handle key;
        public rpc.unicode_string subKey;
        public int options;
        public int desired;
        public rpc.policy_handle result;

        public RegOpenKey(rpc.policy_handle key,
                          rpc.unicode_string subKey,
                          int options,
                          int desired,
                          rpc.policy_handle result) {
            this.key = key;
            this.subKey = subKey;
            this.options = options;
            this.desired = desired;
            this.result = result;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            key.encode(_ndr, _dst);
            subKey.encode(_ndr, _dst);
            _dst.enc_ndr_long(options);
            _dst.enc_ndr_long(desired);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            result.decode(_ndr, _src);
            retval = (int) _src.dec_ndr_long();
        }
    }

    /**
     * Close registry key class
     */
    public static class RegCloseKey extends RegCmd {

        public int getOpnum() {
            return REG_CLOSE;
        }

        public RegCloseKey(rpc.policy_handle handle) {
            this.handle = handle;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            handle.decode(_ndr, _src);
            retval = (int) _src.dec_ndr_long();
        }
    }

    /**
     * perform "registry query value" command class
     * 
     * Properties:
     * 
     * handle - A handle to an open registry key. The key must have been opened
     * with the KEY_QUERY_VALUE access right. For more information, see Registry
     * Key Security and Access Rights. This handle is returned by RegCreateKey,
     * RegCreateKeyTransacted, RegOpenKey, or RegOpenKeyTransacted.
     * 
     * value_name - The name of the registry value. If value_name is NULL or an
     * empty string, "", the function retrieves the type and data for the key's
     * unnamed or default value, if any.
     * 
     * type - A pointer to a variable that receives a code indicating the type
     * of data stored in the specified value. For a list of the possible type
     * codes, see Registry Value Types. The type parameter can be NULL if the
     * type code is not required.
     * 
     * data - A pointer to a buffer that receives the value's data. This
     * parameter can be NULL if the data is not required.
     * 
     * size - A pointer to a variable that specifies the size of the buffer
     * pointed to by the data parameter, in bytes. When the function returns,
     * this variable contains the size of the data copied to data. The size
     * parameter can be NULL only if data is NULL.
     */
    public static class RegQueryValue extends RegCmd {

        public int getOpnum() {
            return REG_INFO;
        }

        public rpc.unicode_string value_name;
        public NdrLong type;
        public byte[] data;
        public NdrLong size;
        public NdrLong length;

        public RegQueryValue(rpc.policy_handle handle,
                             rpc.unicode_string value_name,
                             NdrLong type,
                             byte[] data,
                             NdrLong size,
                             NdrLong length) {
            this.handle = handle;
            this.value_name = value_name;
            this.type = type;
            this.data = data;
            this.size = size;
            this.length = length;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
            value_name.encode(_ndr, _dst);
            _dst.enc_ndr_referent(type, 1);
            if (type != null) {
                type.encode(_ndr, _dst);
            }
            _dst.enc_ndr_referent(data, 1);
            if (data != null) {
                int _datal = length.value;
                int _datas = size.value;
                _dst.enc_ndr_long(_datas);
                _dst.enc_ndr_long(0);
                _dst.enc_ndr_long(_datal);
                int _datai = _dst.index;
                _dst.advance(1 * _datal);

                _dst = _dst.derive(_datai);
                for (int _i = 0; _i < _datal; _i++) {
                    _dst.enc_ndr_small(data[_i]);
                }
            }
            _dst.enc_ndr_referent(size, 1);
            if (size != null) {
                size.encode(_ndr, _dst);
            }
            _dst.enc_ndr_referent(length, 1);
            if (length != null) {
                length.encode(_ndr, _dst);
            }
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            int _typep = _src.dec_ndr_long();
            if (_typep != 0) {
                type.decode(_ndr, _src);
            }
            int _datap = _src.dec_ndr_long();
            if (_datap != 0) {
                int _datas = _src.dec_ndr_long();
                _src.dec_ndr_long();
                int _datal = _src.dec_ndr_long();
                int _datai = _src.index;
                _src.advance(1 * _datal);

                if (data == null) {
                    if (_datas < 0 || _datas > 0xFFFF)
                        throw new NdrException(NdrException.INVALID_CONFORMANCE);
                    data = new byte[_datas];
                }
                _src = _src.derive(_datai);
                for (int _i = 0; _i < _datal; _i++) {
                    data[_i] = (byte) _src.dec_ndr_small();
                }
            }
            int _sizep = _src.dec_ndr_long();
            if (_sizep != 0) {
                size.decode(_ndr, _src);

            }
            int _lengthp = _src.dec_ndr_long();
            if (_lengthp != 0) {
                length.decode(_ndr, _src);

            }
            retval = (int) _src.dec_ndr_long();
        }
    }

    /**
     * Winreg String NDR representation
     * 
     * @author jwebster
     * 
     */
    public static class WinregString extends NdrObject {
        int nameLen;
        int nameSize;
        int namePtr;

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            _dst.enc_ndr_long(0);
            _dst.enc_ndr_long(0);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            nameLen = _src.dec_ndr_short();
            nameSize = _src.dec_ndr_short();
            namePtr = (int) _src.dec_ndr_long();
        }
    }

    /**
     * key information data class
     * 
     * Properties:
     * 
     * reserved - This parameter is reserved and must be NULL.
     * 
     * subKeys - A pointer to a variable that receives the number of subkeys
     * that are contained by the specified key. This parameter can be NULL.
     * 
     * maxSubKeyLen - A pointer to a variable that receives the size of the
     * key's subkey with the longest name, in characters, not including the
     * terminating null character. This parameter can be NULL.
     * 
     * maxClassLen - A pointer to a variable that receives the size of the
     * longest string that specifies a subkey class, in characters. The count
     * returned does not include the terminating null character. This parameter
     * can be NULL.
     * 
     * values - A pointer to a variable that receives the number of values that
     * are associated with the key. This parameter can be NULL.
     * 
     * maxValueNameLen - A pointer to a variable that receives the size of the
     * key's longest value name, in characters. The size does not include the
     * terminating null character. This parameter can be NULL.
     * 
     * maxValueLen - A pointer to a variable that receives the size of the
     * longest data component among the key's values, in bytes. This parameter
     * can be NULL.
     * 
     * securityDescriptor - A pointer to a variable that receives the size of
     * the key's security descriptor, in bytes. This parameter can be NULL.
     * 
     * lastWriteTime - A pointer to a FILETIME structure that receives the last
     * write time. This parameter can be NULL. The function sets the members of
     * the FILETIME structure to indicate the last time that the key or any of
     * its value entries is modified.
     * 
     * @author Joe Webster
     * 
     */
    public static class KeyInfo extends NdrObject {

        public int subKeys;
        public int maxSubKeyLen;
        public int maxClassLen;
        public int values;
        public int maxValueNameLen;
        public int maxValueLen;
        public int securityDescriptor;
        public long lastWriteTime;

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            _dst.enc_ndr_long(0);
            _dst.enc_ndr_long(0);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            subKeys = _src.dec_ndr_long();
            maxSubKeyLen = _src.dec_ndr_long();
            maxClassLen = _src.dec_ndr_long();
            values = _src.dec_ndr_long();
            maxValueNameLen = _src.dec_ndr_long();
            maxValueLen = _src.dec_ndr_long();
            securityDescriptor = _src.dec_ndr_long();
            lastWriteTime = _src.dec_ndr_llong();
        }
    }

    /**
     * perform registry Query Info Key class
     * 
     * Properties:
     * 
     * handle - A handle to an open registry key. The key must have been opened
     * with the KEY_QUERY_VALUE access right.
     * 
     * numSubkeys - number of subkeys that are contained by the specified key.
     * 
     * numValues - number of values that are associated with the key.
     * 
     * @author Joe Webster
     * 
     */
    public static class RegQueryInfoKey extends RegCmd {

        public int getOpnum() {
            return REG_QUERY_KEY;
        }

        public int numSubkeys;
        public int numValues;

        public WinregString classname = new WinregString();
        public KeyInfo keyInfo = new KeyInfo();

        public RegQueryInfoKey(rpc.policy_handle handle) {
            this.handle = handle;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
            keyInfo.encode(_ndr, _dst);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            classname.decode(_ndr, _src);

            // classname is really just a reserved field that should always be
            // NULL
            // with a nameLen or 2 bytes - i.e. it should always have a raw
            // value of 02 00 00 00 00 00 00 00.
            // Unfortunately many existing SMB servers - specifically Samba 3
            // take
            // the MS API spec at face value and simply use 2 NULL DWORDs.
            // IFF you need to check for validity of classname, here is where
            // that would need to happen.

            keyInfo.decode(_ndr, _src);
            numSubkeys = keyInfo.subKeys;
            numValues = keyInfo.values;
            retval = (int) _src.dec_ndr_long();
        }
    }

    public static class GetVersion extends RegCmd {

        public int version;

        public GetVersion(rpc.policy_handle handle) {
            this.handle = handle;
        }

        public int getOpnum() {
            return REG_UNK_1A;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            version = _src.dec_ndr_long();
            retval = (int) _src.dec_ndr_long();
        }

    }

    /**
     * Enum Key class
     * 
     * Properties:
     * 
     * enum_index - The index of the subkey to retrieve. This parameter should
     * be zero for the first call to the RegEnumKeyEx function and then
     * incremented for subsequent calls. Because subkeys are not ordered, any
     * new subkey will have an arbitrary index. This means that the function may
     * return subkeys in any order.
     * 
     * buflen - maxmum length of the name buffer
     * 
     * name - A pointer to a buffer that receives the name of the subkey.
     * 
     * keyclass - A pointer to a buffer that receives the class string of the
     * enumerated subkey.
     * 
     * last_changed_time - A pointer to a variable that receives the time at
     * which the enumerated subkey was last written.
     * 
     * @author Joe Webster
     * 
     */
    public static class EnumKey extends RegCmd {

        public int enum_index;
        private short buflen;
        public String nameStr;
        public rpc.unicode_string name;
        public rpc.unicode_string keyclass;
        public NdrHyper last_changed_time;

        /**
         * constructor on properties
         * 
         * @param enumIndex
         */
        public EnumKey(rpc.policy_handle handle,
                       int enum_index,
                       short buflen,
                       rpc.unicode_string name,
                       rpc.unicode_string keyclass,
                       NdrHyper last_changed_time) {
            this.handle = handle;
            this.enum_index = enum_index;
            this.buflen = buflen;
            this.name = name;
            this.keyclass = keyclass;
            this.last_changed_time = last_changed_time;
        }

        public int getOpnum() {
            return REG_ENUM_KEY;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
            _dst.enc_ndr_long(enum_index);
            name.maximum_length = buflen;
            name.encode(_ndr, _dst);
            keyclass.maximum_length = 1024;
            _dst.enc_ndr_referent(keyclass, 1);
            if (keyclass != null) {
                keyclass.encode(_ndr, _dst);
            }
            _dst.enc_ndr_referent(last_changed_time, 1);
            if (last_changed_time != null) {
                last_changed_time.encode(_ndr, _dst);
            }
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            int _actlen = _src.dec_ndr_short();
            int _maxlen = _src.dec_ndr_short();
            int _namep = _src.dec_ndr_long();
            byte[] data = null;
            if (_namep != 0) {
                int _names = _src.dec_ndr_long();
                _src.dec_ndr_long();
                int _namel = _src.dec_ndr_long();
                int _namei = _src.index;
                _src.advance(2 * _namel);

                if (_names < 0 || _names > 0xFFFF)
                    throw new NdrException(NdrException.INVALID_CONFORMANCE);
                data = new byte[_names];
                _src = _src.derive(_namei);
                for (int _i = 0; _i < _namel; _i++) {
                    data[_i] = (byte) _src.dec_ndr_short();
                }
            }
            nameStr = rpc.unicodeDataToString(data, 0, data.length);
            int _keyclassp = _src.dec_ndr_long();
            if (_keyclassp != 0) {
                _src.dec_ndr_short();
                _src.dec_ndr_short();
                int _kc_namep = _src.dec_ndr_long();
                if (_kc_namep != 0) {
                    _src.dec_ndr_long();
                    _src.dec_ndr_long();
                    int _kcactcnt = _src.dec_ndr_long();
                    _src.advance(2 * _kcactcnt);
                    int _ki = _src.index;
                    _src = _src.derive(_ki);
                }
            }
            int _last_changed_timep = _src.dec_ndr_long();
            if (_last_changed_timep != 0) {
                long lct = _src.dec_ndr_llong();
                last_changed_time = new NdrHyper(lct);
            }
            retval = (int) _src.dec_ndr_long();
        }
    }

    /**
     * Enum Value class
     * 
     * Properties:
     * 
     * handle - A handle to an open registry key. The key must have been opened
     * with the KEY_QUERY_VALUE access right.
     * 
     * enum_index - The index of the value to be retrieved. This parameter
     * should be zero for the first call to the RegEnumValue function and then
     * be incremented for subsequent calls. Because values are not ordered, any
     * new value will have an arbitrary index. This means that the function may
     * return values in any order.
     * 
     * buflen - maxmum length of the name buffer
     * 
     * name - A pointer to a buffer that receives the name of the value,
     * including the terminating null character.
     * 
     * type - pointer to a variable that receives a code indicating the type of
     * data stored in the specified value.
     * 
     * data - A pointer to a buffer that receives the data for the value entry.
     * This parameter can be NULL if the data is not required. If data is NULL
     * and data_size is non-NULL, the function stores the size of the data, in
     * bytes, in the variable pointed to by data. This enables an application to
     * determine the best way to allocate a buffer for the data.
     * 
     * data_size - A pointer to a variable that specifies the size of the buffer
     * pointed to by the data parameter, in bytes. When the function returns,
     * the variable pointed to by the data_size parameter contains the number of
     * bytes stored in the buffer. This parameter can be NULL only if data is
     * NULL.
     * 
     * value_length -
     * 
     * @author Joe Webster
     * 
     */
    public static class EnumValue extends RegCmd {

        public int enum_index;
        private short buflen;
        public String nameStr;
        public rpc.unicode_string name;
        public NdrLong type;
        public byte[] data;
        public NdrLong data_size;
        public NdrLong value_length;

        /**
         * constructor on properties
         * 
         * @param enumIndex
         */
        public EnumValue(rpc.policy_handle handle,
                         int enum_index,
                         short buflen,
                         rpc.unicode_string name,
                         NdrLong type,
                         byte[] data,
                         NdrLong data_size,
                         NdrLong value_length) {
            this.handle = handle;
            this.enum_index = enum_index;
            this.buflen = buflen;
            this.name = name;
            this.type = type;
            this.data = data;
            this.data_size = data_size;
            this.value_length = value_length;
        }

        public int getOpnum() {
            return REG_ENUM_VALUE;
        }

        public void encode(NetworkDataRepresentation _ndr, NdrBuffer _dst)
                throws NdrException {
            handle.encode(_ndr, _dst);
            _dst.enc_ndr_long(enum_index);
            name.maximum_length = buflen;
            name.encode(_ndr, _dst);
            _dst.enc_ndr_referent(type, 1);
            if (type != null) {
                type.encode(_ndr, _dst);
            }
            _dst.enc_ndr_referent(data, 1);
            if (data != null) {
                int _datal = value_length.value;
                int _datas = data_size.value;
                _dst.enc_ndr_long(_datas);
                _dst.enc_ndr_long(0);
                _dst.enc_ndr_long(_datal);
                int _datai = _dst.index;
                _dst.advance(1 * _datal);

                _dst = _dst.derive(_datai);
                for (int _i = 0; _i < _datal; _i++) {
                    _dst.enc_ndr_small(data[_i]);
                }
            }
            _dst.enc_ndr_referent(data_size, 1);
            if (data_size != null) {
                data_size.encode(_ndr, _dst);
            }
            _dst.enc_ndr_referent(value_length, 1);
            if (value_length != null) {
                value_length.encode(_ndr, _dst);
            }
        }

        public void decode(NetworkDataRepresentation _ndr, NdrBuffer _src)
                throws NdrException {
            int _actlen = _src.dec_ndr_short();
            int _maxlen = _src.dec_ndr_short();
            int _namep = _src.dec_ndr_long();
            byte[] _data = null;
            if (_namep != 0) {
                int _names = _src.dec_ndr_long();
                _src.dec_ndr_long();
                int _namel = _src.dec_ndr_long();
                int _namei = _src.index;
                _src.advance(2 * _namel);

                if (_names < 0 || _names > 0xFFFF)
                    throw new NdrException(NdrException.INVALID_CONFORMANCE);
                _data = new byte[_names];
                _src = _src.derive(_namei);
                for (int _i = 0; _i < _namel; _i++) {
                    _data[_i] = (byte) _src.dec_ndr_short();
                }
            }
            nameStr = rpc.unicodeDataToString(_data, 0, _data.length);
            int _typep = _src.dec_ndr_long();
            if (_typep != 0) {
                type.decode(_ndr, _src);
            }
            int _valp = _src.dec_ndr_long();
            if (_valp != 0) {
                int _vals = _src.dec_ndr_long();
                _src.dec_ndr_long();
                int _vall = _src.dec_ndr_long();
                int _vali = _src.index;
                _src.advance(1 * _vall);

                if (_vals < 0 || _vals > 0xFFFF)
                    throw new NdrException(NdrException.INVALID_CONFORMANCE);
                data = new byte[_vals];
                _src = _src.derive(_vali);
                for (int _i = 0; _i < _vall; _i++) {
                    data[_i] = (byte) _src.dec_ndr_small();
                }
            }
            int _data_sizep = _src.dec_ndr_long();
            if (_data_sizep != 0) {
                data_size.decode(_ndr, _src);
            }
            int _value_lengthp = _src.dec_ndr_long();
            if (_value_lengthp != 0) {
                value_length.decode(_ndr, _src);
            }
            retval = (int) _src.dec_ndr_long();
        }
    }

}
